package ch.hevs.kart;

/**
 * Low-level interface in order to get changes from the remote kart represented by a {@link Kart}
 * instance.
 *
 * Implement this interface if you like to get notified about all possible kart status registers
 * that have been send from the kart to the application. If you have only to work with the steering
 * position or the battery level, you can implement the much simpler {@link KartListener} interface.
 * You can implement both interfaces as well.
 *
 * @author Michael Clausen (clm@hevs.ch)
 * @version 2.1 - 2015
 */
@SuppressWarnings("UnusedParameters")
public interface KartStatusRegisterListener {
	/**
	 * This method is called by a {@link Kart} instance to which this listener has been registered
     * for every register that <b>changed</b> its value.
	 * 
	 * @param kart		The Kart instance containing the register that has changed.
	 * @param register	The actual register that has changed.
	 * @param newValue	The value of the register.
	 */
    void statusRegisterHasChanged(Kart kart, KartStatusRegister register, int newValue);
}
