package ch.hevs.kart.comm.discovery;

/**
 * This interface is used by the transport implementations offering discovery services to 
 * 		  inform an object about the retrieval of a new Kart entity.
 * 
 * @author Michael Clausen (clm@hevs.ch)
 * @version 2.1 - 2015
 */
public interface DiscoveryListener {
	/**
	 * The transport layer did (re-)discover a new Kart entity.
	 * 
	 * Note that this method could be called more than once for the very same entity. So you have
	 * to check in your list of entities of the given entity is already present before adding it to
	 * the list.
	 * 
	 * @param discovery	The discovery object that actually discovered the new entity.
	 * @param entity	The Kart entity just discovered.
	 */
	void entityFoundIndication(Discovery discovery, DiscoveryEntity entity);
	
	/**
	 * Discovery errors are reported back to the caller using this interface method.
	 * 
	 * @param discovery	The discovery object that caused the error.
	 * @param error		A textual description of the error.
	 */
	void errorIndication(Discovery discovery, String error);
}
