package ch.hevs.kart.comm.protocol;

import ch.hevs.kart.KartControlRegister;
import ch.hevs.kart.comm.transport.Transport;
import ch.hevs.kart.comm.transport.TransportListener;

/**
 * Protocol abstraction in order to communicate with the Kart. The abstraction is register-
 * 		  based. So basically it is possible to set a register using this interface and get 
 * 		  notified via the ProtocolListener interface about registers that were updated on the 
 * 	      other side.
 * 
 * @author Michael Clausen (clm@hevs.ch)
 * @version 2.1 - 2015
 */
public interface Protocol extends TransportListener {
	/**
	 * Initializes the protocol using the given transport and listener.
	 * 
	 * @param transport	Reference to the transport to be used for communication.
	 * @param listener	Reference to the object to notify about changes.
	 */
	void initialize(Transport transport, ProtocolListener listener);
	
	/**
	 * Can be called in order to close the communication to the Kart.
	 */
	void dispose();
	
	/**
	 * Returns true if the communication protocol is ready to send or receive register 
	 * 		  changes.
	 * 
	 * @return True if ready, false otherwise.
	 */
	boolean isReady();
	
	/**
	 * Sends the given register value to the connected peer.
	 * 
	 * @param register	Kart control register to send.
	 * @param value		The value to send.
	 */
	void setRegisterValue(KartControlRegister register, int value);
}
