package ch.hevs.kart.ui;

import ch.hevs.kart.Kart;
import ch.hevs.kart.KartListener;
import ch.hevs.kart.comm.discovery.DiscoveryEntity;
import android.app.Activity;
import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

/**
 * Basic activity for all kart control activities.
 * 
 * This base class initializes and maintains the connection to the kart and disposes the protected
 * member attribute "kart" in order to enable all subclasses the access to the connected Kart
 * object.
 * 
 * @author Michael Clausen (clm@hevs.ch)
 * @version 2.1 - 2015
 */
public abstract class AbstractKartControlActivity extends Activity implements KartListener {
	
	// Kart intent category to search for (RCA=Remote Control Activity).
	static final String INTENT_CATEGORY = "ch.hevs.kart.RCA";
	
	// Name of the entity passed from the discovery activity to this activity.
	static final String EXTRAS_ENTITY = "KART_COMMUNICATION_ENTITY";
	
	@Override
	public void onCreate(final Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
	
		// Try to get the discovered entity.
		entity = (DiscoveryEntity) getIntent().getExtras().get(EXTRAS_ENTITY);
		if (entity == null) {
			Toast.makeText(this, "Invalid Kart entity!", Toast.LENGTH_LONG).show();
			finish();
		}
	}
	
	@Override
	public void onResume() {
		super.onResume();
		
		// Try to connect to the Kart.
		kart.connect(entity);
		kart.addKartListener(this);
		kart.resetSteering();
	}
	
	@Override
	public void onPause() {
		// Disconnect from the Kart.
		kart.disconnect();
		kart.removeKartListener(this);
		super.onPause();
	}

    /**
     * Sets the enabled flag on the given view and all its subviews.
     *
     * @param view      View to set enabled flag (and all subviews).
     * @param enabled   True in order to enable the view, false in order to disable view.
     */
	protected static void setEnabledForAllSubviews(final View view, final boolean enabled) {
	    view.setEnabled(enabled);
	    view.setFocusable(enabled);

	    if(view instanceof ViewGroup) {
	        ViewGroup vg = (ViewGroup) view;
	        for (int i = 0; i < vg.getChildCount(); i++)
	            setEnabledForAllSubviews(vg.getChildAt(i), enabled);
	    }
	}
	
	// The entity describing the connection to the Kart.
	private DiscoveryEntity entity;

    /**
     * The Kart object. You should use this object for all communication with the kart.
     */
	protected final Kart kart = new Kart();
}
