package ch.hevs.kart.ui;

import java.util.ArrayList;

import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;
import ch.hevs.kart.comm.discovery.DiscoveryEntity;
import ch.hevs.kart.ui.EntityScanActivity.ViewHolder;
import ch.hevs.kart.R;

/*
 * Custom ListView adapter to display Bluetooth devices found through scanning.
 * 
 * @author Christopher Metrailler (mei@hevs.ch)
 * @author Michael Clausen (clm@hevs.ch)
 * @version 2.1 - 2015
 */
class EntityListAdapter extends BaseAdapter {

	@SuppressWarnings("serial")
	class EntityArrayList extends ArrayList<DiscoveryEntity> {
		public boolean containsSimilar(DiscoveryEntity entity) {
			for (DiscoveryEntity e: this) {
				if (entity.equals(e))
					return true;
			}
			return false;
		}
	}
	
	private final EntityArrayList entities;
	private final LayoutInflater inflater;

	public EntityListAdapter(Activity a) {
		entities = new EntityArrayList();
		inflater = a.getLayoutInflater();
	}

	public void addEntity(DiscoveryEntity entity) {
		if (entity != null && !entities.contains(entity) && !entities.containsSimilar(entity)) {
			entities.add(entity);
			notifyDataSetChanged();
		}
	}

	public ArrayList<DiscoveryEntity> getEntities() {
		return entities;
	}
	
	public DiscoveryEntity getEntity(int position) {
		return entities.get(position);
	}

	public void clear() {
		entities.clear();
	}

	@Override
	public int getCount() {
		return entities.size();
	}

	@Override
	public Object getItem(int i) {
		return entities.get(i);
	}

	@Override
	public long getItemId(int i) {
		return i;
	}

	@Override
	public View getView(int i, View view, ViewGroup viewGroup) {
		ViewHolder viewHolder;
		// General ListView optimization code.
		if (view == null) {
			view = inflater.inflate(R.layout.kart_discover_list_item_layout, null);
			viewHolder = new ViewHolder();
			viewHolder.deviceAddress = (TextView) view
					.findViewById(R.id.device_address);
			viewHolder.deviceName = (TextView) view
					.findViewById(R.id.device_name);
			viewHolder.icon = (ImageView) view.findViewById(R.id.icon);
			view.setTag(viewHolder);
		} else {
			viewHolder = (ViewHolder) view.getTag();
		}

		DiscoveryEntity entity = entities.get(i);
		final String entityName = entity.getFriendlyName();
		
		if (entity.getTransportImplementation() != null && 
			!entity.getSupportedProtocols().isEmpty()) {
			viewHolder.icon.setVisibility(View.VISIBLE);
			view.setEnabled(true);
		} else {
			viewHolder.icon.setVisibility(View.GONE);
			view.setEnabled(false);
		}

		if (entityName != null && entityName.length() > 0)
			viewHolder.deviceName.setText(entityName);
		else
			viewHolder.deviceName.setText(R.string.unknown_device);

		viewHolder.deviceAddress.setText(entity.getDescription());

		return view;
	}
}