package ch.hevs.kart.ui;

import java.util.List;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Intent;
import android.content.pm.ResolveInfo;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import ch.hevs.kart.R;
import ch.hevs.kart.comm.CommunicationSetup;
import ch.hevs.kart.comm.discovery.Discovery;
import ch.hevs.kart.comm.discovery.DiscoveryEntity;
import ch.hevs.kart.comm.discovery.DiscoveryListener;

/**
 * Activity for scanning and displaying available Kart devices. This or a subclass should be the
 * start activity of the kart application.
 * 
 * @author Christopher Metrailler (mei@hevs.ch)
 * @author Michael Clausen (clm@hevs.ch)
 * @version 2.1 - 2015
 */
@SuppressLint("Registered")
public class EntityScanActivity extends Activity implements DiscoveryListener, OnItemClickListener {

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		// Get all available RCA Activities.
		Intent intent = new Intent(Intent.ACTION_RUN);
		intent.addCategory(AbstractKartControlActivity.INTENT_CATEGORY);
		rcActivities = getPackageManager().queryIntentActivities(intent, 0);

		// ListView Activity setup
		setContentView(R.layout.kart_discover_layout);
		foundDevices = (ListView)findViewById(R.id.list);
		foundDevices.setEmptyView(findViewById(R.id.empty));
		foundDevices.setOnItemClickListener(this);
		
		// Initializes list view adapter.
		adapter = new EntityListAdapter(this);
		foundDevices.setAdapter(adapter);
		registerForContextMenu(foundDevices);
	}

	@Override
	protected void onResume() {
		super.onResume();
		scanDevice(true);
	}

	@Override
	protected void onPause() {
		if (scanningInProgress)
			scanDevice(false);
		adapter.clear();
		super.onPause();
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		if (scanningInProgress)
			scanDevice(false);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.kart_discovery_menu, menu);
		if (!scanningInProgress) {
			menu.findItem(R.id.menu_stop).setVisible(false);
			menu.findItem(R.id.menu_scan).setVisible(true);
			menu.findItem(R.id.menu_refresh).setActionView(null);
		} else {
			menu.findItem(R.id.menu_stop).setVisible(true);
			menu.findItem(R.id.menu_scan).setVisible(false);
			menu.findItem(R.id.menu_refresh).setActionView(
					R.layout.kart_discovery_actionbar);
		}
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menu_scan:
			adapter.clear();
			scanDevice(true);
			break;
		case R.id.menu_stop:
			scanDevice(false);
			break;
		}
		return true;
	}

	private void scanDevice(final boolean enable) {
		if (enable) {
			scanningInProgress = true;
			CommunicationSetup.startDiscovery(this, this);
		} else {
			CommunicationSetup.stopDiscovery();
			scanningInProgress = false;
		}
		invalidateOptionsMenu();
	}

	@Override
	public void onItemClick(AdapterView<?> parent, View view, int position,
			long id) {
		if (!selectEntityAtIndex(position))
			return;

		if (rcActivities.isEmpty()) {
            Toast.makeText(this, R.string.error_no_activity, Toast.LENGTH_LONG).show();
        } else if (rcActivities.size() == 1)
			startControlActivity(rcActivities.get(0).activityInfo.name);
		else
			openContextMenu(foundDevices);
	}

	@Override
	public void entityFoundIndication(Discovery discovery, DiscoveryEntity entity) {
		adapter.addEntity(entity);
	}

	@Override
	public void errorIndication(Discovery discovery, String error) {
		Toast.makeText(this, error, Toast.LENGTH_LONG).show();
	}

	@Override
	public void onCreateContextMenu(ContextMenu menu, View view, ContextMenuInfo menuInfo) {
		super.onCreateContextMenu(menu, view, menuInfo);
		// Display available rc Activities
		for(int i = 0; i < rcActivities.size(); i++) {
			final ResolveInfo info = rcActivities.get(i);
			String label = info.activityInfo.loadLabel(getPackageManager()).toString();
			menu.add(0, i, i, label);
		}
	}

	@Override
	public boolean onContextItemSelected(MenuItem item) {
		startControlActivity(rcActivities.get(item.getItemId()).activityInfo.name);
		return true;
	}

	protected void startControlActivity(String className) {
		final Intent intent = new Intent();
		intent.setClassName(this, className);
		intent.putExtra(AbstractKartControlActivity.EXTRAS_ENTITY, entity);
		startActivity(intent);
	}

	protected boolean selectEntityAtIndex(final int index) {
        entity = adapter.getEntity(index);
        return entity != null;
    }

	private ListView foundDevices;
	private EntityListAdapter adapter;
	private boolean scanningInProgress = false;
	private DiscoveryEntity entity;
	private List<ResolveInfo> rcActivities;

	static class ViewHolder {
		TextView deviceName;
		TextView deviceAddress;
		ImageView icon;
	}
}