package ch.hevs.utils.math;

/**
 * Offers static methods in order to limit values to a given range.
 *
 * @author Michael Clausen (clm@hevs.ch)
 * @version 2.1 - 2015
 */
public class Limits {
    /**
     * Generic limit function. Limits the given value to the given range (min, max).
     *
     * @param value     The input value to limit.
     * @param min       The minimal value.
     * @param max       The maximal allowed value.
     * @param <T>       The type of the value and limits have to implement the Comparable interface.
     * @return          Value limited to the given limits (min, max).
     */
    public static <T extends Comparable<T>> T limit(final T value, final T min, final T max) {
        if (value.compareTo(max) > 0)
            return max;
        if (value.compareTo(min) < 0)
            return min;
        return value;
	}
}
